/*
    DnsUpdate
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    In addition, as a special exception, the copyright holder of this
    program gives permission to link the code of its release of DnsUpdate
    with the OpenSSL project's "OpenSSL" library (or with modified
    versions of it that use the same license as the "OpenSSL" library);
    and distribute the linked executables. You must obey the GNU General
    Public License in all respects for all of the code used other than
    "OpenSSL". If you modify this file, you may extend this exception to
    your version of the file, but you are not obliged to do so. If you
    do not wish to do so, delete this exception statement from your version.

    Martin Storsj
    martin@martin.st
*/

#include <stdio.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <stdlib.h>
#include <sys/time.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

static char *base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

char get64char(unsigned char c) {
	return base64[c];
}

char *base64_encode(unsigned char *in, int inlen) {
	char *out = malloc(1+4*((inlen+2)/3));
	unsigned int temp;
	int pos, n;
	char *outptr = out;

	pos = 0;
	while (pos<inlen) {
		temp = 0;
		n = 1;
		temp |= *in++<<16;
		pos++;
		if (pos<inlen) {
			temp |= *in++<<8;
			pos++;
			n++;
		}
		if (pos<inlen) {
			temp |= *in++;
			pos++;
			n++;
		}
		*outptr++ = get64char((temp>>18)&0x3F);
		*outptr++ = get64char((temp>>12)&0x3F);
		if (n>1) *outptr++ = get64char((temp>>6)&0x3F);
		else *outptr++ = '=';
		if (n>2) *outptr++ = get64char(temp&0x03F);
		else *outptr++ = '=';
	}

	*outptr = '\0';

	return out;
}

char *copy(char *str) {
	char *out = malloc(strlen(str)+1);
	strcpy(out,str);
	return out;
}

void readconfig(FILE *in, char **login, char **password, char **host, char **ip) {
	char line[500];
	char *pos;
	while (fgets(line,sizeof(line),in)) {
		if (line[strlen(line)-1]=='\n') line[strlen(line)-1] = '\0';
		if (!strncmp(line,"login",strlen("login"))) {
			pos = strchr(line,'=');
			if (pos!=NULL) {
				pos++;
				*login = copy(pos);
			}
		} else if (!strncmp(line,"password",strlen("password"))) {
			pos = strchr(line,'=');
			if (pos!=NULL) {
				pos++;
				*password = copy(pos);
			}
		} else if (!strncmp(line,"host",strlen("host"))) {
			pos = strchr(line,'=');
			if (pos!=NULL) {
				pos++;
				*host = copy(pos);
			}
		} else if (!strncmp(line,"ip",strlen("ip"))) {
			pos = strchr(line,'=');
			if (pos!=NULL) {
				pos++;
				*ip = copy(pos);
			}
		}
	}
}

int main(int argc, char *argv[]) {
	struct sockaddr_in addr;
	struct hostent *h;
	int sock;
	SSL *ssl;
	SSL_CTX *ctx;
	char *server = "dynamic.zoneedit.com";
	char *login = NULL;
	char *password = NULL;
	char *host = NULL;
	char *ip = NULL;
	char request[2048];
	char *auth;
	char buffer[4096], *ptr, *ptr2;
	int len;
	int header = 1;
	int i;
	FILE *in;

	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i],"--file")) {
			i++;
			if (!strcmp(argv[i],"-")) {
				readconfig(stdin,&login,&password,&host,&ip);
			} else {
				in = fopen(argv[i],"r");
				readconfig(in,&login,&password,&host,&ip);
				fclose(in);
			}
		} else if (!strcmp(argv[i],"--login")) {
			i++;
			login = copy(argv[i]);
		} else if (!strcmp(argv[i],"--password")) {
			i++;
			password = copy(argv[i]);
		} else if (!strcmp(argv[i],"--host")) {
			i++;
			host = copy(argv[i]);
		} else if (!strcmp(argv[i],"--ip")) {
			i++;
			ip = copy(argv[i]);
		}
	}

	sprintf(request,"%s:%s",login,password);
	auth = base64_encode((unsigned char*)request,strlen(request));
	if (ip!=NULL) {
		sprintf(request,"GET /auth/dynamic.html?host=%s&dnsto=%s HTTP/1.0\nHost: %s\nAuthorization: Basic %s\nConnection: close\n\n",host,ip,server,auth);
	} else {
		sprintf(request,"GET /auth/dynamic.html?host=%s HTTP/1.0\nHost: %s\nAuthorization: Basic %s\nConnection: close\n\n",host,server,auth);
	}
	free(auth);
	free(login);
	free(password);
	free(host);
	free(ip);

	SSL_load_error_strings();
	SSL_library_init();

	if ((sock = socket(AF_INET,SOCK_STREAM,0))<0) {
		perror("socket");
		return 1;
	}

	if ((h = gethostbyname(server))==NULL) {
		herror(server);
		close(sock);
		return 1;
	}

	addr.sin_family = AF_INET;
	addr.sin_port = htons(443);
	addr.sin_addr = *((struct in_addr*)h->h_addr);
	memset(&addr.sin_zero,0,8);

	if (connect(sock,(struct sockaddr*)&addr,sizeof(addr))) {
		perror("connect");
		close(sock);
		return 1;
	}

	ctx = SSL_CTX_new(SSLv23_client_method());
	if (ctx==NULL) {
		ERR_print_errors_fp(stderr);
		close(sock);
		return 1;
	}

	ssl = SSL_new(ctx);
	if (ssl==NULL) {
		ERR_print_errors_fp(stderr);
		SSL_CTX_free(ctx);
		close(sock);
		return 1;
	}

	SSL_set_fd(ssl,sock);
	if (SSL_connect(ssl)<0) {
		ERR_print_errors_fp(stderr);
		SSL_free(ssl);
		SSL_CTX_free(ctx);
		close(sock);
		return 1;
	}

	SSL_write(ssl,request,strlen(request));

	while ((len = SSL_read(ssl,buffer,sizeof(buffer)))>0) {
		buffer[len] = '\0';
		ptr = buffer;
		if (header) {
			ptr = strstr(buffer,"\r\n\r\n");
			if (ptr!=NULL) ptr += 4;
			ptr2 = strstr(buffer,"\n\n");
			if (ptr2!=NULL) ptr2 += 2;
			if (ptr==NULL) {
				ptr = ptr2;
			}
			if (ptr==NULL) ptr = buffer+strlen(buffer);
			else {
				if (ptr2!=NULL) {
					if (ptr>ptr2)
						ptr = ptr2;
				}
				header = 0;
			}
		}
		printf("%s",ptr);
	}

	SSL_shutdown(ssl);

	SSL_free(ssl);
	SSL_CTX_free(ctx);
	close(sock);

	return 0;
}

